//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdio.h>      // Standard functions

// Configuration Functions
void Configure_SPI(void);
void Configure_ProcPorts(void);
void Configure_TLC5971(void);

// Delay Functions
void Delay_ms(int x);
void SixUsDelay(void);  // Used for programming LED Driver

// Communication Functions
void SendData32(int x);
void SendData8(int x);
void Program (int ctrl, int L1BG, int L1R2B, int L2GR, int L3BG, int L3R4B, int L4GR);

// Individual constant light LED functions
void Brown(void);
void Orange(void);
void White(void);
void Yellow(void);
void Violet(void);
void Lime(void);
void Cyan(void);

void BLU2(void);
void BLU2GRN2(void);
void BLU2GRN4(void);
void BLU2GRN6(void);
void BLU2GRN8(void);
void BLU2RED2(void);
void BLU2RED4(void);
void BLU2RED6(void);
void BLU2RED8(void);
void BLU4(void);
void BLU4GRN2(void);
void BLU4GRN4(void);
void BLU4GRN6(void);
void BLU4GRN8(void);
void BLU4RED2(void);
void BLU4RED4(void);
void BLU4RED6(void);
void BLU4RED8(void);
void BLU6(void);
void BLU6GRN2(void);
void BLU6GRN4(void);
void BLU6GRN6(void);
void BLU6GRN8(void);
void BLU6RED2(void);
void BLU6RED4(void);
void BLU6RED6(void);
void BLU6RED8(void);
void BLU8(void);
void BLU8GRN2(void);
void BLU8GRN4(void);
void BLU8GRN6(void);
void BLU8GRN8(void);
void BLU8RED2(void);
void BLU8RED4(void);
void BLU8RED6(void);
void BLU8RED8(void);
void GRN2(void);
void GRN2RED2(void);
void GRN2RED4(void);
void GRN2RED6(void);
void GRN2RED8(void);
void GRN4(void);
void GRN4RED2(void);
void GRN4RED4(void);
void GRN4RED6(void);
void GRN4RED8(void);
void GRN6(void);
void GRN6RED2(void);
void GRN6RED4(void);
void GRN6RED6(void);
void GRN6RED8(void);
void GRN8(void);
void GRN8RED2(void);
void GRN8RED4(void);
void GRN8RED6(void);
void GRN8RED8(void);
void Off(void);
void RED2(void);
void RED4(void);
void RED6(void);
void RED8(void);

// Multiple constant light LED functions
void AllGreen(void);
void AllWhite(void);
void AllYellow(void);
void AllBrown(void);
void AllOff(void);
void AllRed(void);

void OrangeLimeCyanViolet(void);
void RedGreenBlueOff(void);
void OrangeYellowBrownViolet(void);

void OrangeBrownBrownBrown(void);
void BrownOrangeBrownBrown(void);
void BrownBrownOrangeBrown(void);
void BrownBrownBrownOrange(void);

// LED light show functions
void AllBlinkYellow(void);
void AllFadeInBrown(void);
void AllFadeOutBrown(void);
void AllFadeOutYellow(void);
void AllBrownOneOrangeToRight(void);
void AllBrownOneOrangeToLeft(void);
void AllBrownOneOrangeSlide(void);


// Global Variables
DELAY = 5000;
long int DECODER = 0x25;     // bits 223-218 decoder: 6b'100101
long int FC = 0x12;          // 5 bits
                             //// bit 217 (OUTTMG): 1 OUTXn off/on rising edge, 0 OUTXn off/on falling edge
                             //// bit 216 (EXTCLK): 1 SCKI GS clock, 0 internal GS clock 
                             //// bit 215 (TMGRST): 1 enabled GS counter reset OUTXn off, 0 disabled
                             //// bit 214 (DSPRPT): 1 enable auto display repeat, 0 disabled
                             //// bit 213 (BLANK): 1 enable blanking, 0 disabled
long int BC = 0x1FFFFF;      // 21 bits
                             //// bit 212-206: BC data for blue
                             //// bit 205-199: BC data for green
                             //// bit 198-192: BC data for red
int thirty_two = 32;
int LED_CNT = 4;

void main(void){
  // M8C_EnableGInt ;     // Uncomment this line to enable Global Interrupts
                          // Would be needed if using timer interrupt
 
  Configure_SPI();        // Configure SPI
  Configure_ProcPorts();  // Configure Ports for communcation
  Configure_TLC5971();    // Configure TLC5971

  while(1) {
  	AllBlinkYellow();
	AllFadeInBrown();
	AllBrownOneOrangeSlide();
	AllFadeOutBrown();
  }
}

//-------------------------
// Configuration Functions
//-------------------------
void Configure_SPI(void) {
  // SPIM_1_EnableInt();                                    /* Enable Interrupt */
  SPIM_1_Start(SPIM_1_SPIM_MODE_0 | SPIM_1_SPIM_MSB_FIRST);  //Set to mode 0 & shift MSB first
	                                                         //Data changes on falling edge, rising
															 //edge latches data
}

void Configure_ProcPorts(void){
  // Configure GPIO for strong output
  PRT2DM0 = 0x01;
  PRT2DM1 = 0x00;
  PRT2DM2 = 0x00;

  // Enable the differential driver, only needed for TLC5970 design
  PRT2DR = 0x01;
}

void Configure_TLC5971(void) {
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  SendData8(0x00);      // LED1: Blue MSB
  SendData8(0x00);      // LED1: Blue LSB
  SendData8(0x00);      // LED1: Green LSB
  SendData8(0x00);      // LED1: Green LSB
  SendData8(0x00);      // LED1: Red LSB
  SendData8(0x00);      // LED1: Red LSB
  
  SendData8(0x00);      // LED2: Blue MSB
  SendData8(0x00);      // LED2: Blue LSB
  SendData8(0x00);      // LED2: Green LSB
  SendData8(0x00);      // LED2: Green LSB
  SendData8(0x00);      // LED2: Red LSB
  SendData8(0x00);      // LED2: Red LSB

  SendData8(0x00);      // LED3: Blue MSB
  SendData8(0x00);      // LED3: Blue LSB
  SendData8(0x00);      // LED3: Green LSB
  SendData8(0x00);      // LED3: Green LSB
  SendData8(0x00);      // LED3: Red LSB
  SendData8(0x00);      // LED3: Red LSB

  SendData8(0x00);      // LED3: Blue MSB
  SendData8(0x00);      // LED3: Blue LSB
  SendData8(0x00);      // LED3: Green LSB
  SendData8(0x00);      // LED3: Green LSB
  SendData8(0x00);      // LED3: Red LSB
  SendData8(0x00);      // LED3: Red LSB
}

//-------------------------
// Delay Functions
//-------------------------
void Delay_ms(int x){
  int y = 0;
  int z = 0;
	
  while ( y < x ) {
    while ( z < 62 ) {   // This is is 1ms
	  z++;
    }
    y++;
  }
}

void SixUsDelay(void){
  // SysClk = 24MHz
  // NOP takes 4 clock cycles
  // 6us = 1/24MHz * 4 * 32
//  int y;
//  for (y = 1; y<= 32; y++) {
//    asm("NOP");
//  }
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
  asm("NOP");
}

//-------------------------
// Communication Functions
//-------------------------
void SendData32 (int x) {
  int y;
  while( ! (SPIM_1_bReadStatus() & SPIM_1_SPIM_TX_BUFFER_EMPTY ) );
  for (y = 24; y >= 0; y-=8){
    int program = x >> y;
	x = x & 0xff;
	SPIM_1_SendTxData(x);
  }
}

void SendData8 (int x) {
  int y;
  while( ! (SPIM_1_bReadStatus() & SPIM_1_SPIM_TX_BUFFER_EMPTY ) );
  SPIM_1_SendTxData(x);
}

void Program (int ctrl, int L1BG, int L1R2B, int L2GR, int L3BG, int L3R4B, int L4GR){
  SendData32(ctrl);
  SendData32(L1BG);
  SendData32(L1R2B);
  SendData32(L2GR);
  SendData32(L3BG);
  SendData32(L3R4B);
  SendData32(L4GR);
  Delay_ms(DELAY); // Time to shift data into latches
}

//-------------------------
// Individual constant light LED functions
//-------------------------
void Brown(void){
  SendData8(0x10);      // Blue MSB
  SendData8(0x10);      // Blue LSB
  SendData8(0x45);      // Green LSB
  SendData8(0x45);      // Green LSB
  SendData8(0x8b);      // Red LSB
  SendData8(0x8b);      // Red LSB
}
void Orange(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void Violet(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void Yellow(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void White(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void Lime(void){
  SendData8(0x00);      // LED2: Blue MSB
  SendData8(0x00);      // LED2: Blue LSB
  SendData8(0xff);      // LED2: Green LSB
  SendData8(0xff);      // LED2: Green LSB
  SendData8(0x80);      // LED2: Red LSB
  SendData8(0x00);      // LED2: Red LSB
}
void Cyan(void){
  SendData8(0xff);      // LED3: Blue MSB
  SendData8(0xff);      // LED3: Blue LSB
  SendData8(0xff);      // LED3: Green LSB
  SendData8(0xff);      // LED3: Green LSB
  SendData8(0x00);      // LED3: Red LSB
  SendData8(0x00);      // LED3: Red LSB
}
void BLU2(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU2GRN2(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU2GRN4(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU2GRN6(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xbf);      // Green MSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU2GRN8(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green MSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU2RED2(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU2RED4(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU2RED6(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU2RED8(void){
  SendData8(0x40);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU4(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU4GRN2(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU4GRN4(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU4GRN6(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU4GRN8(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green MSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU4RED2(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU4RED4(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU4RED6(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU4RED8(void){
  SendData8(0x80);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green MSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red MSB
  SendData8(0xff);      // Red LSB
}
void BLU6(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU6GRN2(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU6GRN4(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU6GRN6(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0xbf);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU6GRN8(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU6RED2(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU6RED4(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU6RED6(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU6RED8(void){
  SendData8(0xbf);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU8(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU8GRN2(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU8GRN4(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU8GRN6(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0xbf);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU8GRN8(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU8RED2(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU8RED4(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void BLU8RED6(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void BLU8RED8(void){
  SendData8(0xff);      // Blue MSB
  SendData8(0xff);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN2(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN2RED2(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN2RED4(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN2RED6(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN2RED8(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x40);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN4(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN4RED2(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN4RED4(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN4RED6(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN4RED8(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x80);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN6(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xbf);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN6RED2(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xbf);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN6RED4(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xbf);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN6RED6(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xbf);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN6RED8(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xbf);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN8(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN8RED2(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN8RED4(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void GRN8RED6(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void GRN8RED8(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void Off(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void RED2(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x40);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void RED4(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x80);      // Red LSB
  SendData8(0x00);      // Red LSB
}
void RED6(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xbf);      // Red LSB
  SendData8(0xff);      // Red LSB
}
void RED8(void){
  SendData8(0x00);      // Blue MSB
  SendData8(0x00);      // Blue LSB
  SendData8(0x00);      // Green LSB
  SendData8(0x00);      // Green LSB
  SendData8(0xff);      // Red LSB
  SendData8(0xff);      // Red LSB
}

//-------------------------
// Multiple constant light LED functions
//-------------------------
void RedGreenBlueOff(void){
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  RED8();
  GRN8();
  BLU8();
  Off();
}
void OrangeYellowBrownViolet(void){
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  Orange();
  Yellow();
  Violet();
  Brown();
}
void AllYellow(void){
  int y;

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  for (y =0; y < LED_CNT; y++) {
    Yellow();
  }
}
void AllBrown(void){
  int y;

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  for (y =0; y < LED_CNT; y++) {
	Brown();
  }
}
void AllWhite(void){
  int y;

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  for (y =0; y < LED_CNT; y++) {
	White();
  }
}
void AllOff(void){
  int y;

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  for (y =0; y < LED_CNT; y++) {
	Off();
  }
}

void OrangeLimeCyanViolet(void){
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  Orange();
  Lime();
  Cyan();
  Violet();
}
void AllRed(void){
  int y;

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  for (y =0; y < LED_CNT; y++) {
	RED8();
  }
}
void AllGreen(void){
  int y;

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  
  for (y =0; y < LED_CNT; y++) {
	  GRN8();
  }
}
void OrangeBrownBrownBrown(void){
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  Orange();
  Brown();
  Brown();
  Brown();
}
void BrownOrangeBrownBrown(void){
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  Brown();
  Orange();
  Brown();
  Brown();
}
void BrownBrownOrangeBrown(void){
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  Brown();
  Brown();
  Orange();
  Brown();
}
void BrownBrownBrownOrange(void){
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  Brown();
  Brown();
  Brown();
  Orange();
}

//-------------------------
// LED light show functions
//-------------------------
void AllBlinkYellow(void){
  int y = 10000;
  AllYellow();
  Delay_ms(y);
  AllFadeOutYellow();
  AllOff();
  Delay_ms(y);
  AllYellow();
  Delay_ms(y);
  AllFadeOutYellow();
  AllOff();
  Delay_ms(y);
  AllYellow();
  Delay_ms(y);
  AllFadeOutYellow();
  AllOff();
  Delay_ms(y);
  AllYellow();
  Delay_ms(y);
  AllFadeOutYellow();
  AllOff();
  Delay_ms(y);
  AllYellow();
  Delay_ms(y);
  AllFadeOutYellow();
  AllOff();
  Delay_ms(y);
  AllYellow();
  Delay_ms(y);
  AllFadeOutYellow();
  AllOff();
  Delay_ms(y);
}

void AllFadeInBrown(void){
  int y = 10000;
  AllOff();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x44);      // 5-bit FC
  SendData8(0x08);      // 21-bit BC
  SendData8(0x10);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x48);      // 5-bit FC
  SendData8(0x10);      // 21-bit BC
  SendData8(0x20);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x4c);      // 5-bit FC
  SendData8(0x18);      // 21-bit BC
  SendData8(0x30);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x50);      // 5-bit FC
  SendData8(0x20);      // 21-bit BC
  SendData8(0x40);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x54);      // 5-bit FC
  SendData8(0x28);      // 21-bit BC
  SendData8(0x50);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x58);      // 5-bit FC
  SendData8(0x30);      // 21-bit BC
  SendData8(0x60);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5c);      // 5-bit FC
  SendData8(0x38);      // 21-bit BC
  SendData8(0x70);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);

  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
}

void AllFadeOutBrown(void){
  int y = 10000;
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5c);      // 5-bit FC
  SendData8(0x38);      // 21-bit BC
  SendData8(0x70);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x58);      // 5-bit FC
  SendData8(0x30);      // 21-bit BC
  SendData8(0x60);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x54);      // 5-bit FC
  SendData8(0x28);      // 21-bit BC
  SendData8(0x50);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x50);      // 5-bit FC
  SendData8(0x20);      // 21-bit BC
  SendData8(0x40);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x4c);      // 5-bit FC
  SendData8(0x18);      // 21-bit BC
  SendData8(0x30);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x48);      // 5-bit FC
  SendData8(0x10);      // 21-bit BC
  SendData8(0x20);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x44);      // 5-bit FC
  SendData8(0x08);      // 21-bit BC
  SendData8(0x10);      ///////////////////
  Brown();
  Brown();
  Brown();
  Brown();
  Delay_ms(y);
  
  AllOff();
  Delay_ms(y);
}

void AllFadeOutYellow(void){
  int y = 6000;
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5f);      // 5-bit FC
  SendData8(0xff);      // 21-bit BC
  SendData8(0xff);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x5c);      // 5-bit FC
  SendData8(0x38);      // 21-bit BC
  SendData8(0x70);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x58);      // 5-bit FC
  SendData8(0x30);      // 21-bit BC
  SendData8(0x60);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x54);      // 5-bit FC
  SendData8(0x28);      // 21-bit BC
  SendData8(0x50);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x50);      // 5-bit FC
  SendData8(0x20);      // 21-bit BC
  SendData8(0x40);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x4c);      // 5-bit FC
  SendData8(0x18);      // 21-bit BC
  SendData8(0x30);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x48);      // 5-bit FC
  SendData8(0x10);      // 21-bit BC
  SendData8(0x20);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  SendData8(0x96);      // 6-bit Decoder
  SendData8(0x44);      // 5-bit FC
  SendData8(0x08);      // 21-bit BC
  SendData8(0x10);      ///////////////////
  Yellow();
  Yellow();
  Yellow();
  Yellow();
  Delay_ms(y);
  
  AllOff();
  Delay_ms(y);
}

void AllBrownOneOrangeToRight(void){
  int y = 18000;

  OrangeBrownBrownBrown();
  Delay_ms(y);

  BrownOrangeBrownBrown();
  Delay_ms(y);

  BrownBrownOrangeBrown();
  Delay_ms(y);

  BrownBrownBrownOrange();
  Delay_ms(y);
}

void AllBrownOneOrangeToLeft(void){
  int y = 18000;

  BrownBrownBrownOrange();
  Delay_ms(y);

  BrownBrownOrangeBrown();
  Delay_ms(y);

  BrownOrangeBrownBrown();
  Delay_ms(y);

  OrangeBrownBrownBrown();  
  Delay_ms(y);
}

void AllBrownOneOrangeSlide(void){
  int y = 18000;
  AllBrown();
  Delay_ms(y);
  
  AllBrownOneOrangeToRight();
  Delay_ms(y);
  
  AllBrown();
  Delay_ms(y);
  
  AllBrownOneOrangeToLeft();
  Delay_ms(y);

  AllBrown();
  Delay_ms(y);
  
  AllBrownOneOrangeToRight();
  Delay_ms(y);
  
  AllBrown();
  Delay_ms(y);
}
